@@ -1,5 +1,6 @@
 from __future__ import annotations
 
+import warnings
 from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Tuple
 
 from gradio.blocks import BlockContext
@@ -10,22 +11,6 @@ set_documentation_group("layout")
 if TYPE_CHECKING:  # Only import for type checking (is False at runtime).
     from gradio.components import Component
 
-valid_colors = [
-    "red",
-    "green",
-    "blue",
-    "yellow",
-    "purple",
-    "teal",
-    "orange",
-    "cyan",
-    "lime",
-    "pink",
-    "black",
-    "grey",
-    "gray",
-]
-
 
 @document()
 class Row(BlockContext):
@@ -37,8 +22,19 @@ class Row(BlockContext):
                 gr.Image("lion.jpg")
                 gr.Image("tiger.jpg")
         demo.launch()
+    Guides: controlling_layout
     """
 
+    def __init__(
+        self, *, visible: bool = True, elem_id: Optional[str] = None, **kwargs
+    ):
+        """
+        Parameters:
+            visible: If False, row will be hidden.
+            elem_id: An optional string that is assigned as the id of this component in the HTML DOM. Can be used for targeting CSS styles.
+        """
+        super().__init__(visible=visible, elem_id=elem_id)
+
     def get_config(self):
         return {"type": "row", **super().get_config()}
 
@@ -54,48 +50,67 @@ class Row(BlockContext):
     def style(
         self,
         equal_height: Optional[bool] = None,
-        mobile_collapse: Optional[bool] = True,
+        mobile_collapse: Optional[bool] = None,
     ):
+        """
+        Styles the Row.
+        Parameters:
+            equal_height: If True, makes every child element have equal height
+            mobile_collapse: DEPRECATED.
+        """
         if equal_height is not None:
             self._style["equal_height"] = equal_height
         if mobile_collapse is not None:
-            self._style["mobile_collapse"] = mobile_collapse
-
+            warnings.warn("mobile_collapse is no longer supported.", DeprecationWarning)
         return self
 
 
 @document()
 class Column(BlockContext):
     """
-    Column is a layout element within Blocks that renders all children vertically.
+    Column is a layout element within Blocks that renders all children vertically. The widths of columns can be set through the `scale` and `min_width` parameters.
+    If a certain scale results in a column narrower than min_width, the min_width parameter will win.
     Example:
         with gradio.Blocks() as demo:
             with gradio.Row():
-                with gradio.Column():
+                with gradio.Column(scale=1):
                     text1 = gr.Textbox()
                     text2 = gr.Textbox()
-                with gradio.Column():
+                with gradio.Column(scale=4):
                     btn1 = gr.Button("Button 1")
                     btn2 = gr.Button("Button 2")
+    Guides: controlling_layout
     """
 
     def __init__(
         self,
-        visible: bool = True,
+        *,
+        scale: int = 1,
+        min_width: int = 320,
         variant: str = "default",
+        visible: bool = True,
+        elem_id: Optional[str] = None,
+        **kwargs,
     ):
         """
         Parameters:
-            visible: If False, column will be hidden but included in the Blocks config file (its visibility can later be updated).
+            scale: relative width compared to adjacent Columns. For example, if Column A has scale=2, and Column B has scale=1, A will be twice as wide as B.
+            min_width: minimum pixel width of Column, will wrap if not sufficient screen space to satisfy this value. If a certain scale value results in a column narrower than min_width, the min_width parameter will be respected first.
             variant: column type, 'default' (no background) or 'panel' (gray background color and rounded corners)
+            visible: If False, column will be hidden.
+            elem_id: An optional string that is assigned as the id of this component in the HTML DOM. Can be used for targeting CSS styles.
         """
+        self.scale = scale
+        self.min_width = min_width
         self.variant = variant
-        super().__init__(visible=visible)
+        super().__init__(visible=visible, elem_id=elem_id)
 
     def get_config(self):
         return {
             "type": "column",
             "variant": self.variant,
+            "scale": self.scale,
+            "min_width": self.min_width,
             **super().get_config(),
         }
 
@@ -111,29 +126,26 @@ class Column(BlockContext):
         }
 
 
-@document()
 class Tabs(BlockContext):
     """
-    Tabs is a layout element within Blocks that can contain multiple TabItem's. Each
-    TabItem gets rendered as a individual tab. The TabItem's must be nested within the
-    Tabs context.
-    Example:
-        with gradio.Blocks() as demo:
-            with gradio.Tabs():
-                with gradio.TabItem("Lion"):
-                    gr.Image("lion.jpg")
-                    gr.Button("New Lion")
-                with gradio.TabItem("Tiger"):
-                    gr.Image("tiger.jpg")
-                    gr.Button("New Tiger")
+    Tabs is a layout element within Blocks that can contain multiple "Tab" Components.
     """
 
-    def __init__(self, selected: Optional[int | str] = None, **kwargs):
+    def __init__(
+        self,
+        *,
+        selected: Optional[int | str] = None,
+        visible: bool = True,
+        elem_id: Optional[str] = None,
+        **kwargs,
+    ):
         """
         Parameters:
             selected: The currently selected tab. Must correspond to an id passed to the one of the child TabItems. Defaults to the first TabItem.
+            visible: If False, Tabs will be hidden.
+            elem_id: An optional string that is assigned as the id of this component in the HTML DOM. Can be used for targeting CSS styles.
         """
-        super().__init__(**kwargs)
+        super().__init__(visible=visible, elem_id=elem_id, **kwargs)
         self.selected = selected
 
     def get_config(self):
@@ -159,23 +171,32 @@ class Tabs(BlockContext):
 
 
 class TabItem(BlockContext):
-    """
-    TabItem is a layout element that must be defined within a Tabs context. The
-    components defined within the TabItem will be rendered within a tab.
-    """
+    expected_parent = Tabs
 
-    def __init__(self, label: str, id: Optional[int | str] = None, **kwargs):
+    def __init__(
+        self,
+        label: str,
+        *,
+        id: Optional[int | str] = None,
+        elem_id: Optional[str] = None,
+        **kwargs,
+    ):
         """
         Parameters:
             label: The visual label for the tab
             id: An optional identifier for the tab, required if you wish to control the selected tab from a predict function.
+            elem_id: An optional string that is assigned as the id of this component in the HTML DOM. Can be used for targeting CSS styles.
         """
-        super().__init__(**kwargs)
+        super().__init__(elem_id=elem_id, **kwargs)
         self.label = label
         self.id = id
 
     def get_config(self):
-        return {"label": self.label, "id": self.id, **super().get_config()}
+        return {
+            "label": self.label,
+            "id": self.id,
+            **super().get_config(),
+        }
 
     def select(self, fn: Callable, inputs: List[Component], outputs: List[Component]):
         """
@@ -188,6 +209,27 @@ class TabItem(BlockContext):
         self.set_event_trigger("select", fn, inputs, outputs)
 
 
+@document()
+class Tab(TabItem):
+    """
+    Tab is a layout element. Components defined within the Tab will be visible when this tab is selected tab.
+    Example:
+        with gradio.Blocks() as demo:
+            with gradio.Tab("Lion"):
+                gr.Image("lion.jpg")
+                gr.Button("New Lion")
+            with gradio.Tab("Tiger"):
+                gr.Image("tiger.jpg")
+                gr.Button("New Tiger")
+    Guides: controlling_layout
+    """
+
+    pass
+
+
+Tab = TabItem  # noqa: F811
+
+
 @document()
 class Group(BlockContext):
     """
@@ -199,6 +241,20 @@ class Group(BlockContext):
             gr.Textbox(label="Last")
     """
 
+    def __init__(
+        self,
+        *,
+        visible: bool = True,
+        elem_id: Optional[str] = None,
+        **kwargs,
+    ):
+        """
+        Parameters:
+            visible: If False, group will be hidden.
+            elem_id: An optional string that is assigned as the id of this component in the HTML DOM. Can be used for targeting CSS styles.
+        """
+        super().__init__(visible=visible, elem_id=elem_id, **kwargs)
+
     def get_config(self):
         return {"type": "group", **super().get_config()}
 
@@ -236,6 +292,20 @@ class Box(BlockContext):
             gr.Textbox(label="Last")
     """
 
+    def __init__(
+        self,
+        *,
+        visible: bool = True,
+        elem_id: Optional[str] = None,
+        **kwargs,
+    ):
+        """
+        Parameters:
+            visible: If False, box will be hidden.
+            elem_id: An optional string that is assigned as the id of this component in the HTML DOM. Can be used for targeting CSS styles.
+        """
+        super().__init__(visible=visible, elem_id=elem_id, **kwargs)
+
     def get_config(self):
         return {"type": "box", **super().get_config()}
 
@@ -261,3 +331,56 @@ class Box(BlockContext):
         if border is not None:
             self._style["border"] = border
         return self
+
+
+class Form(BlockContext):
+    def get_config(self):
+        return {"type": "form", **super().get_config()}
+
+
+@document()
+class Accordion(BlockContext):
+    """
+    Accordion is a layout element which can be toggled to show/hide the contained content.
+    Example:
+        with gradio.Accordion("See Details"):
+            gr.Markdown("lorem ipsum")
+    """
+
+    def __init__(
+        self,
+        label,
+        *,
+        open: bool = True,
+        visible: bool = True,
+        elem_id: Optional[str] = None,
+        **kwargs,
+    ):
+        """
+        Parameters:
+            label: name of accordion section.
+            open: if True, accordion is open by default.
+            elem_id: An optional string that is assigned as the id of this component in the HTML DOM. Can be used for targeting CSS styles.
+        """
+        self.label = label
+        self.open = open
+        super().__init__(visible=visible, elem_id=elem_id, **kwargs)
+
+    def get_config(self):
+        return {
+            "type": "accordion",
+            "open": self.open,
+            "label": self.label,
+            **super().get_config(),
+        }
+
+    @staticmethod
+    def update(
+        open: Optional[bool] = None,
+        visible: Optional[bool] = None,
+    ):
+        return {
+            "visible": visible,
+            "open": open,
+            "__type__": "update",
+        }
