@@ -1,21 +1,26 @@
 import base64
+import json
 import mimetypes
 import os
+import pathlib
 import shutil
+import subprocess
 import tempfile
 import warnings
 from io import BytesIO
 
 import numpy as np
 import requests
-from PIL import Image, ImageOps
+from ffmpy import FFmpeg, FFprobe, FFRuntimeError
+from PIL import Image, ImageOps, PngImagePlugin
 
-from gradio import encryptor
+from gradio import encryptor, utils
 
 with warnings.catch_warnings():
     warnings.simplefilter("ignore")  # Ignore pydub warning if ffmpeg is not installed
     from pydub import AudioSegment
 
+
 #########################
 # IMAGE PRE-PROCESSING
 #########################
@@ -25,20 +30,11 @@ def decode_base64_to_image(encoding):
     return Image.open(BytesIO(base64.b64decode(image_encoded)))
 
 
-def get_url_or_file_as_bytes(path):
-    try:
-        return requests.get(path).content
-    except (requests.exceptions.MissingSchema, requests.exceptions.InvalidSchema):
-        with open(path, "rb") as f:
-            return f.read()
-
-
-def encode_url_or_file_to_base64(path):
-    try:
-        requests.get(path)
-        return encode_url_to_base64(path)
-    except (requests.exceptions.MissingSchema, requests.exceptions.InvalidSchema):
-        return encode_file_to_base64(path)
+def encode_url_or_file_to_base64(path, encryption_key=None):
+    if utils.validate_url(path):
+        return encode_url_to_base64(path, encryption_key=encryption_key)
+    else:
+        return encode_file_to_base64(path, encryption_key=encryption_key)
 
 
 def get_mimetype(filename):
@@ -74,8 +70,10 @@ def encode_file_to_base64(f, encryption_key=None):
         )
 
 
-def encode_url_to_base64(url):
+def encode_url_to_base64(url, encryption_key=None):
     encoded_string = base64.b64encode(requests.get(url).content)
+    if encryption_key:
+        encoded_string = encryptor.decrypt(encryption_key, encoded_string)
     base64_str = str(encoded_string, "utf-8")
     mimetype = get_mimetype(url)
     return (
@@ -91,10 +89,51 @@ def encode_plot_to_base64(plt):
     return "data:image/png;base64," + base64_str
 
 
+def download_to_file(url, dir=None):
+    file_suffix = os.path.splitext(url)[1]
+    file_obj = tempfile.NamedTemporaryFile(delete=False, suffix=file_suffix, dir=dir)
+    with requests.get(url, stream=True) as r:
+        with open(file_obj.name, "wb") as f:
+            shutil.copyfileobj(r.raw, f)
+    return file_obj
+
+
+def save_array_to_file(image_array, dir=None):
+    pil_image = Image.fromarray(_convert(image_array, np.uint8, force_copy=False))
+    file_obj = tempfile.NamedTemporaryFile(delete=False, suffix=".png", dir=dir)
+    pil_image.save(file_obj)
+    return file_obj
+
+
+def save_pil_to_file(pil_image, dir=None):
+    file_obj = tempfile.NamedTemporaryFile(delete=False, suffix=".png", dir=dir)
+    pil_image.save(file_obj)
+    return file_obj
+
+
+def encode_pil_to_base64(pil_image):
+    with BytesIO() as output_bytes:
+
+        # Copy any text-only metadata
+        use_metadata = False
+        metadata = PngImagePlugin.PngInfo()
+        for key, value in pil_image.info.items():
+            if isinstance(key, str) and isinstance(value, str):
+                metadata.add_text(key, value)
+                use_metadata = True
+
+        pil_image.save(
+            output_bytes, "PNG", pnginfo=(metadata if use_metadata else None)
+        )
+        bytes_data = output_bytes.getvalue()
+    base64_str = str(base64.b64encode(bytes_data), "utf-8")
+    return "data:image/png;base64," + base64_str
+
+
 def encode_array_to_base64(image_array):
     with BytesIO() as output_bytes:
-        PIL_image = Image.fromarray(_convert(image_array, np.uint8, force_copy=False))
-        PIL_image.save(output_bytes, "PNG")
+        pil_image = Image.fromarray(_convert(image_array, np.uint8, force_copy=False))
+        pil_image.save(output_bytes, "PNG")
         bytes_data = output_bytes.getvalue()
     base64_str = str(base64.b64encode(bytes_data), "utf-8")
     return "data:image/png;base64," + base64_str
@@ -104,7 +143,8 @@ def resize_and_crop(img, size, crop_type="center"):
     """
     Resize and crop an image to fit the specified size.
     args:
-        size: `(width, height)` tuple.
+        size: `(width, height)` tuple. Pass `None` for either width or height
+        to only crop and resize the other.
         crop_type: can be 'top', 'middle' or 'bottom', depending on this
             value, the image will cropped getting the 'top/left', 'middle' or
             'bottom/right' of the image to fit the size.
@@ -117,7 +157,13 @@ def resize_and_crop(img, size, crop_type="center"):
         center = (0.5, 0.5)
     else:
         raise ValueError
-    return ImageOps.fit(img, size, centering=center)
+
+    resize = list(size)
+    if size[0] is None:
+        resize[0] = img.size[0]
+    if size[1] is None:
+        resize[1] = img.size[1]
+    return ImageOps.fit(img, resize, centering=center)
 
 
 ##################
@@ -126,7 +172,14 @@ def resize_and_crop(img, size, crop_type="center"):
 
 
 def audio_from_file(filename, crop_min=0, crop_max=100):
-    audio = AudioSegment.from_file(filename)
+    try:
+        audio = AudioSegment.from_file(filename)
+    except FileNotFoundError as e:
+        error_message = str(e)
+        if "ffprobe" in error_message:
+            print(
+                "Please install `ffmpeg` in your system to use non-WAV audio file formats."
+            )
     if crop_min != 0 or crop_max != 100:
         audio_start = len(audio) * crop_min / 100
         audio_end = len(audio) * crop_max / 100
@@ -138,13 +191,45 @@ def audio_from_file(filename, crop_min=0, crop_max=100):
 
 
 def audio_to_file(sample_rate, data, filename):
+    data = convert_to_16_bit_wav(data)
     audio = AudioSegment(
         data.tobytes(),
         frame_rate=sample_rate,
         sample_width=data.dtype.itemsize,
         channels=(1 if len(data.shape) == 1 else data.shape[1]),
     )
-    audio.export(filename, format="wav")
+    audio.export(filename, format="wav").close()
+
+
+def convert_to_16_bit_wav(data):
+    # Based on: https://docs.scipy.org/doc/scipy/reference/generated/scipy.io.wavfile.write.html
+    if data.dtype == np.float32:
+        warnings.warn(
+            "Audio data is not in 16-bit integer format."
+            "Trying to convert to 16-bit int format."
+        )
+        data = data / np.abs(data).max()
+        data = data * 32767
+        data = data.astype(np.int16)
+    elif data.dtype == np.int32:
+        warnings.warn(
+            "Audio data is not in 16-bit integer format."
+            "Trying to convert to 16-bit int format."
+        )
+        data = data / 65538
+        data = data.astype(np.int16)
+    elif data.dtype == np.int16:
+        pass
+    elif data.dtype == np.uint8:
+        warnings.warn(
+            "Audio data is not in 16-bit integer format."
+            "Trying to convert to 16-bit int format."
+        )
+        data = data * 257 - 32768
+        data = data.astype(np.int16)
+    else:
+        raise ValueError("Audio data cannot be converted to " "16-bit int format.")
+    return data
 
 
 ##################
@@ -158,20 +243,26 @@ def decode_base64_to_binary(encoding):
     return base64.b64decode(data), extension
 
 
-def decode_base64_to_file(encoding, encryption_key=None, file_path=None):
+def decode_base64_to_file(
+    encoding, encryption_key=None, file_path=None, dir=None, prefix=None
+):
+    if dir is not None:
+        os.makedirs(dir, exist_ok=True)
     data, extension = decode_base64_to_binary(encoding)
-    prefix = None
-    if file_path is not None:
+    if file_path is not None and prefix is None:
         filename = os.path.basename(file_path)
         prefix = filename
         if "." in filename:
             prefix = filename[0 : filename.index(".")]
             extension = filename[filename.index(".") + 1 :]
     if extension is None:
-        file_obj = tempfile.NamedTemporaryFile(delete=False, prefix=prefix)
+        file_obj = tempfile.NamedTemporaryFile(delete=False, prefix=prefix, dir=dir)
     else:
         file_obj = tempfile.NamedTemporaryFile(
-            delete=False, prefix=prefix, suffix="." + extension
+            delete=False,
+            prefix=prefix,
+            suffix="." + extension,
+            dir=dir,
         )
     if encryption_key is not None:
         data = encryptor.encrypt(encryption_key, data)
@@ -180,17 +271,55 @@ def decode_base64_to_file(encoding, encryption_key=None, file_path=None):
     return file_obj
 
 
-def create_tmp_copy_of_file(file_path):
+def create_tmp_copy_of_file_or_url(file_path_or_url: str, dir=None):
+    try:
+        response = requests.get(file_path_or_url, stream=True)
+        if file_path_or_url.find("/"):
+            new_file_path = file_path_or_url.rsplit("/", 1)[1]
+        else:
+            new_file_path = "file.txt"
+        with open(new_file_path, "wb") as out_file:
+            shutil.copyfileobj(response.raw, out_file)
+        del response
+    except (requests.exceptions.MissingSchema, requests.exceptions.InvalidSchema):
+        return create_tmp_copy_of_file(file_path_or_url, dir)
+
+
+def dict_or_str_to_json_file(jsn, dir=None):
+    if dir is not None:
+        os.makedirs(dir, exist_ok=True)
+
+    file_obj = tempfile.NamedTemporaryFile(
+        delete=False, suffix=".json", dir=dir, mode="w+"
+    )
+    if isinstance(jsn, str):
+        jsn = json.loads(jsn)
+    json.dump(jsn, file_obj)
+    file_obj.flush()
+    return file_obj
+
+
+def file_to_json(file_path):
+    return json.load(open(file_path))
+
+
+def create_tmp_copy_of_file(file_path, dir=None):
+    if dir is not None:
+        os.makedirs(dir, exist_ok=True)
+
     file_name = os.path.basename(file_path)
     prefix, extension = file_name, None
     if "." in file_name:
         prefix = file_name[0 : file_name.index(".")]
         extension = file_name[file_name.index(".") + 1 :]
     if extension is None:
-        file_obj = tempfile.NamedTemporaryFile(delete=False, prefix=prefix)
+        file_obj = tempfile.NamedTemporaryFile(delete=False, prefix=prefix, dir=dir)
     else:
         file_obj = tempfile.NamedTemporaryFile(
-            delete=False, prefix=prefix, suffix="." + extension
+            delete=False,
+            prefix=prefix,
+            suffix="." + extension,
+            dir=dir,
         )
     shutil.copy2(file_path, file_obj.name)
     return file_obj
@@ -311,13 +440,7 @@ def _convert(image, dtype, force_copy=False, uniform=False):
             Output image array. Has the same kind as `a`.
         """
         kind = a.dtype.kind
-        if n > m and a.max() < 2 ** m:
-            mnew = int(np.ceil(m / 2) * 2)
-            if mnew > m:
-                dtype = "int{}".format(mnew)
-            else:
-                dtype = "uint{}".format(mnew)
-            n = int(np.ceil(n / 2) * 2)
+        if n > m and a.max() < 2**m:
             return a.astype(_dtype_bits(kind, m))
         elif n == m:
             return a.copy() if copy else a
@@ -334,11 +457,11 @@ def _convert(image, dtype, force_copy=False, uniform=False):
             # exact upscale to a multiple of `n` bits
             if copy:
                 b = np.empty(a.shape, _dtype_bits(kind, m))
-                np.multiply(a, (2 ** m - 1) // (2 ** n - 1), out=b, dtype=b.dtype)
+                np.multiply(a, (2**m - 1) // (2**n - 1), out=b, dtype=b.dtype)
                 return b
             else:
                 a = a.astype(_dtype_bits(kind, m, a.dtype.itemsize), copy=False)
-                a *= (2 ** m - 1) // (2 ** n - 1)
+                a *= (2**m - 1) // (2**n - 1)
                 return a
         else:
             # upscale to a multiple of `n` bits,
@@ -346,12 +469,12 @@ def _convert(image, dtype, force_copy=False, uniform=False):
             o = (m // n + 1) * n
             if copy:
                 b = np.empty(a.shape, _dtype_bits(kind, o))
-                np.multiply(a, (2 ** o - 1) // (2 ** n - 1), out=b, dtype=b.dtype)
+                np.multiply(a, (2**o - 1) // (2**n - 1), out=b, dtype=b.dtype)
                 b //= 2 ** (o - m)
                 return b
             else:
                 a = a.astype(_dtype_bits(kind, o, a.dtype.itemsize), copy=False)
-                a *= (2 ** o - 1) // (2 ** n - 1)
+                a *= (2**o - 1) // (2**n - 1)
                 a //= 2 ** (o - m)
                 return a
 
@@ -481,3 +604,53 @@ def _convert(image, dtype, force_copy=False, uniform=False):
     image = _scale(image, 8 * itemsize_in, 8 * itemsize_out, copy=False)
     image += imin_out
     return image.astype(dtype_out)
+
+
+def ffmpeg_installed() -> bool:
+    return shutil.which("ffmpeg") is not None
+
+
+def video_is_playable(video_filepath: str) -> bool:
+    """Determines if a video is playable in the browser.
+
+    A video is playable if it has a playable container and codec.
+        .mp4 -> h264
+        .webm -> vp9
+        .ogg -> theora
+    """
+    try:
+        container = pathlib.Path(video_filepath).suffix.lower()
+        probe = FFprobe(
+            global_options="-show_format -show_streams -select_streams v -print_format json",
+            inputs={video_filepath: None},
+        )
+        output = probe.run(stderr=subprocess.PIPE, stdout=subprocess.PIPE)
+        output = json.loads(output[0])
+        video_codec = output["streams"][0]["codec_name"]
+        return (container, video_codec) in [
+            (".mp4", "h264"),
+            (".ogg", "theora"),
+            (".webm", "vp9"),
+        ]
+    # If anything goes wrong, assume the video can be played to not convert downstream
+    except (FFRuntimeError, IndexError, KeyError):
+        return True
+
+
+def convert_video_to_playable_mp4(video_path: str) -> str:
+    """Convert the video to mp4. If something goes wrong return the original video."""
+    try:
+        output_path = pathlib.Path(video_path).with_suffix(".mp4")
+        with tempfile.NamedTemporaryFile(delete=False) as tmp_file:
+            shutil.copy2(video_path, tmp_file.name)
+            # ffmpeg will automatically use h264 codec (playable in browser) when converting to mp4
+            ff = FFmpeg(
+                inputs={str(tmp_file.name): None},
+                outputs={str(output_path): None},
+                global_options="-y -loglevel quiet",
+            )
+            ff.run()
+    except FFRuntimeError as e:
+        print(f"Error converting video to browser-playable format {str(e)}")
+        output_path = video_path
+    return str(output_path)
