@@ -2,17 +2,21 @@
 
 from __future__ import annotations
 
+import asyncio
 import inspect
 import io
+import mimetypes
 import os
 import posixpath
 import secrets
 import traceback
-import urllib
+from collections import defaultdict
 from copy import deepcopy
 from pathlib import Path
 from typing import Any, List, Optional, Type
+from urllib.parse import urlparse
 
+import fastapi
 import orjson
 import pkg_resources
 from fastapi import Depends, FastAPI, HTTPException, Request, status
@@ -22,11 +26,16 @@ from fastapi.security import OAuth2PasswordRequestForm
 from fastapi.templating import Jinja2Templates
 from jinja2.exceptions import TemplateNotFound
 from pydantic import BaseModel
-from starlette.concurrency import run_in_threadpool
 from starlette.responses import RedirectResponse
+from starlette.websockets import WebSocket, WebSocketState
 
 import gradio
-from gradio import encryptor, queueing
+from gradio import encryptor, utils
+from gradio.documentation import document, set_documentation_group
+from gradio.exceptions import Error
+from gradio.queue import Estimation, Event
+
+mimetypes.init()
 
 STATIC_TEMPLATE_LIB = pkg_resources.resource_filename("gradio", "templates/")
 STATIC_PATH_LIB = pkg_resources.resource_filename("gradio", "templates/frontend/static")
@@ -65,7 +74,7 @@ class QueuePushBody(BaseModel):
 class PredictBody(BaseModel):
     session_hash: Optional[str]
     data: Any
-    fn_index: int = 0
+    fn_index: Optional[int]
 
 
 ###########
@@ -82,6 +91,9 @@ class App(FastAPI):
         self.tokens = None
         self.auth = None
         self.blocks: Optional[gradio.Blocks] = None
+        self.state_holder = {}
+        self.iterators = defaultdict(dict)
+
         super().__init__(**kwargs)
 
     def configure_app(self, blocks: gradio.Blocks) -> None:
@@ -110,7 +122,6 @@ class App(FastAPI):
             allow_methods=["*"],
             allow_headers=["*"],
         )
-        app.state_holder = {}
 
         @app.get("/user")
         @app.get("/user/")
@@ -162,6 +173,8 @@ class App(FastAPI):
         @app.head("/", response_class=HTMLResponse)
         @app.get("/", response_class=HTMLResponse)
         def main(request: Request, user: str = Depends(get_current_user)):
+            mimetypes.add_type("application/javascript", ".js")
+
             if app.auth is None or not (user is None):
                 config = app.blocks.config
             else:
@@ -209,8 +222,10 @@ class App(FastAPI):
             else:
                 return FileResponse(app.blocks.favicon_path)
 
-        @app.get("/file/{path:path}", dependencies=[Depends(login_check)])
-        def file(path):
+        @app.get("/file={path:path}", dependencies=[Depends(login_check)])
+        def file(path: str):
+            if utils.validate_url(path):
+                return RedirectResponse(url=path, status_code=status.HTTP_302_FOUND)
             if (
                 app.blocks.encrypt
                 and isinstance(app.blocks.examples, str)
@@ -222,19 +237,22 @@ class App(FastAPI):
                 return FileResponse(
                     io.BytesIO(file_data), attachment_filename=os.path.basename(path)
                 )
+            elif Path(app.cwd).resolve() in Path(path).resolve().parents or any(
+                Path(temp_dir).resolve() in Path(path).resolve().parents
+                for temp_dir in app.blocks.temp_dirs
+            ):
+                return FileResponse(
+                    Path(path).resolve(), headers={"Accept-Ranges": "bytes"}
+                )
             else:
-                if Path(app.cwd).resolve() in Path(path).resolve().parents:
-                    return FileResponse(Path(path).resolve())
+                raise ValueError(
+                    f"File cannot be fetched: {path}, perhaps because "
+                    f"it is not in any of {app.blocks.temp_dirs}"
+                )
 
-        @app.post("/api/queue/push/", dependencies=[Depends(login_check)])
-        async def queue_push(body: QueuePushBody):
-            job_hash, queue_position = queueing.push(body)
-            return {"hash": job_hash, "queue_position": queue_position}
-
-        @app.post("/api/queue/status/", dependencies=[Depends(login_check)])
-        async def queue_status(body: QueueStatusBody):
-            status, data = queueing.get_status(body.hash)
-            return {"status": status, "data": data}
+        @app.get("/file/{path:path}", dependencies=[Depends(login_check)])
+        def file_deprecated(path: str):
+            return file(path)
 
         async def run_predict(
             body: PredictBody, username: str = Depends(get_current_user)
@@ -247,20 +265,28 @@ class App(FastAPI):
                         if getattr(block, "stateful", False)
                     }
                 session_state = app.state_holder[body.session_hash]
+                iterators = app.iterators[body.session_hash]
             else:
                 session_state = {}
+                iterators = {}
+            raw_input = body.data
+            fn_index = body.fn_index
             try:
-                raw_input = body.data
-                fn_index = body.fn_index
                 output = await app.blocks.process_api(
-                    fn_index, raw_input, username, session_state
+                    fn_index, raw_input, username, session_state, iterators
                 )
+                iterator = output.pop("iterator", None)
+                if hasattr(body, "session_hash"):
+                    app.iterators[body.session_hash][fn_index] = iterator
+                if isinstance(output, Error):
+                    raise output
             except BaseException as error:
-                if app.blocks.show_error:
-                    traceback.print_exc()
-                    return JSONResponse(content={"error": str(error)}, status_code=500)
-                else:
-                    raise error
+                show_error = app.blocks.show_error or isinstance(error, Error)
+                traceback.print_exc()
+                return JSONResponse(
+                    content={"error": str(error) if show_error else None},
+                    status_code=500,
+                )
             return output
 
         @app.post("/api/{api_name}", dependencies=[Depends(login_check)])
@@ -282,6 +308,44 @@ class App(FastAPI):
                     )
             return await run_predict(body=body, username=username)
 
+        @app.websocket("/queue/join")
+        async def join_queue(websocket: WebSocket):
+            if app.blocks._queue.server_path is None:
+                print(f"WS: {str(websocket.url)}")
+                app_url = get_server_url_from_ws_url(str(websocket.url))
+                print(f"Server URL: {app_url}")
+                app.blocks._queue.set_url(app_url)
+
+            await websocket.accept()
+            event = Event(websocket)
+            rank = app.blocks._queue.push(event)
+            if rank is None:
+                await app.blocks._queue.send_message(event, {"msg": "queue_full"})
+                await event.disconnect()
+                return
+            estimation = app.blocks._queue.get_estimation()
+            await app.blocks._queue.send_estimation(event, estimation, rank)
+            while True:
+                await asyncio.sleep(60)
+                if websocket.application_state == WebSocketState.DISCONNECTED:
+                    return
+
+        @app.get(
+            "/queue/status",
+            dependencies=[Depends(login_check)],
+            response_model=Estimation,
+        )
+        async def get_queue_status():
+            return app.blocks._queue.get_estimation()
+
+        @app.get(
+            "/startup-events",
+            dependencies=[Depends(login_check)],
+        )
+        async def startup_events():
+            app.blocks.startup_events()
+            return True
+
         return app
 
 
@@ -321,3 +385,52 @@ def get_types(cls_set: List[Type]):
                 types.append(line.split("value (")[1].split(")")[0])
         docset.append(doc_lines[1].split(":")[-1])
     return docset, types
+
+
+def get_server_url_from_ws_url(ws_url: str):
+    ws_url = urlparse(ws_url)
+    scheme = "http" if ws_url.scheme == "ws" else "https"
+    port = f":{ws_url.port}" if ws_url.port else ""
+    return f"{scheme}://{ws_url.hostname}{port}{ws_url.path.replace('queue/join', '')}"
+
+
+set_documentation_group("routes")
+
+
+@document()
+def mount_gradio_app(
+    app: fastapi.FastAPI,
+    blocks: gradio.Blocks,
+    path: str,
+    gradio_api_url: Optional[str] = None,
+) -> fastapi.FastAPI:
+    """Mount a gradio.Blocks to an existing FastAPI application.
+
+    Parameters:
+        app: The parent FastAPI application.
+        blocks: The blocks object we want to mount to the parent app.
+        path: The path at which the gradio application will be mounted.
+        gradio_api_url: The full url at which the gradio app will run. This is only needed if deploying to Huggingface spaces of if the websocket endpoints of your deployed app are on a different network location than the gradio app. If deploying to spaces, set gradio_api_url to 'http://localhost:7860/'
+    Example:
+        from fastapi import FastAPI
+        import gradio as gr
+        app = FastAPI()
+        @app.get("/")
+        def read_main():
+            return {"message": "This is your main app"}
+        io = gr.Interface(lambda x: "Hello, " + x + "!", "textbox", "textbox")
+        app = gr.mount_gradio_app(app, io, path="/gradio")
+        # Then run `uvicorn run:app` from the terminal and navigate to http://localhost:8000/gradio.
+    """
+
+    gradio_app = App.create_app(blocks)
+
+    @app.on_event("startup")
+    async def start_queue():
+        if gradio_app.blocks.enable_queue:
+            if gradio_api_url:
+                gradio_app.blocks._queue.set_url(gradio_api_url)
+            gradio_app.blocks.startup_events()
+
+    app.mount(path, gradio_app)
+    return app
