@@ -7,4 +7,4 @@ function make_script(src) {
     script.src = src;
     document.head.appendChild(script);
 }
-make_script("https://gradio.s3-us-west-2.amazonaws.com/3.1.1/assets/index.8ea4571f.js");
+make_script("https://gradio.s3-us-west-2.amazonaws.com/3.4/assets/index.8bf4b92e.js");
