@@ -1,11 +1,12 @@
 import gradio
 
 XRAY_CONFIG = {
+    "version": "3.1.8b\n",
     "mode": "blocks",
     "dev_mode": True,
     "components": [
         {
-            "id": 1,
+            "id": 23,
             "type": "markdown",
             "props": {
                 "value": "<h1>Detect Disease From Scan</h1>\n<p>With this model you can lorem ipsum</p>\n<ul>\n<li>ipsum 1</li>\n<li>ipsum 2</li>\n</ul>\n",
@@ -15,7 +16,7 @@ XRAY_CONFIG = {
             },
         },
         {
-            "id": 2,
+            "id": 24,
             "type": "checkboxgroup",
             "props": {
                 "choices": ["Covid", "Malaria", "Lung Cancer"],
@@ -27,30 +28,19 @@ XRAY_CONFIG = {
                 "style": {},
             },
         },
+        {"id": 25, "type": "tabs", "props": {"visible": True, "style": {}}},
         {
-            "id": 3,
-            "type": "tabs",
-            "props": {
-                "visible": True,
-                "style": {},
-            },
-        },
-        {
-            "id": 4,
+            "id": 26,
             "type": "tabitem",
-            "props": {
-                "label": "X-ray",
-                "visible": True,
-                "style": {},
-            },
+            "props": {"label": "X-ray", "visible": True, "style": {}},
         },
         {
-            "id": 5,
+            "id": 27,
             "type": "row",
             "props": {"type": "row", "visible": True, "style": {}},
         },
         {
-            "id": 6,
+            "id": 28,
             "type": "image",
             "props": {
                 "image_mode": "RGB",
@@ -65,12 +55,17 @@ XRAY_CONFIG = {
             },
         },
         {
-            "id": 7,
+            "id": 29,
             "type": "json",
-            "props": {"show_label": True, "name": "json", "visible": True, "style": {}},
+            "props": {
+                "show_label": True,
+                "name": "json",
+                "visible": True,
+                "style": {},
+            },
         },
         {
-            "id": 8,
+            "id": 30,
             "type": "button",
             "props": {
                 "value": "Run",
@@ -81,21 +76,17 @@ XRAY_CONFIG = {
             },
         },
         {
-            "id": 9,
+            "id": 31,
             "type": "tabitem",
-            "props": {
-                "label": "CT Scan",
-                "visible": True,
-                "style": {},
-            },
+            "props": {"label": "CT Scan", "visible": True, "style": {}},
         },
         {
-            "id": 10,
+            "id": 32,
             "type": "row",
             "props": {"type": "row", "visible": True, "style": {}},
         },
         {
-            "id": 11,
+            "id": 33,
             "type": "image",
             "props": {
                 "image_mode": "RGB",
@@ -110,12 +101,17 @@ XRAY_CONFIG = {
             },
         },
         {
-            "id": 12,
+            "id": 34,
             "type": "json",
-            "props": {"show_label": True, "name": "json", "visible": True, "style": {}},
+            "props": {
+                "show_label": True,
+                "name": "json",
+                "visible": True,
+                "style": {},
+            },
         },
         {
-            "id": 13,
+            "id": 35,
             "type": "button",
             "props": {
                 "value": "Run",
@@ -126,7 +122,7 @@ XRAY_CONFIG = {
             },
         },
         {
-            "id": 14,
+            "id": 36,
             "type": "textbox",
             "props": {
                 "lines": 1,
@@ -138,59 +134,70 @@ XRAY_CONFIG = {
                 "style": {},
             },
         },
+        {
+            "id": 37,
+            "type": "form",
+            "props": {"type": "form", "visible": True, "style": {}},
+        },
+        {
+            "id": 38,
+            "type": "form",
+            "props": {"type": "form", "visible": True, "style": {}},
+        },
     ],
     "theme": "default",
     "css": None,
-    "enable_queue": False,
+    "title": "Gradio",
+    "is_space": False,
+    "enable_queue": None,
+    "show_error": False,
     "layout": {
-        "id": 0,
+        "id": 22,
         "children": [
-            {"id": 1},
-            {"id": 2},
+            {"id": 23},
+            {"id": 37, "children": [{"id": 24}]},
             {
-                "id": 3,
+                "id": 25,
                 "children": [
                     {
-                        "id": 4,
+                        "id": 26,
                         "children": [
-                            {"id": 5, "children": [{"id": 6}, {"id": 7}]},
-                            {"id": 8},
+                            {"id": 27, "children": [{"id": 28}, {"id": 29}]},
+                            {"id": 30},
                         ],
                     },
                     {
-                        "id": 9,
+                        "id": 31,
                         "children": [
-                            {"id": 10, "children": [{"id": 11}, {"id": 12}]},
-                            {"id": 13},
+                            {"id": 32, "children": [{"id": 33}, {"id": 34}]},
+                            {"id": 35},
                         ],
                     },
                 ],
             },
-            {"id": 14},
+            {"id": 38, "children": [{"id": 36}]},
         ],
     },
     "dependencies": [
         {
-            "targets": [8],
+            "targets": [30],
             "trigger": "click",
-            "inputs": [2, 6],
-            "outputs": [7],
+            "inputs": [24, 28],
+            "outputs": [29],
             "backend_fn": True,
             "js": None,
-            "status_tracker": None,
             "queue": None,
             "api_name": None,
             "scroll_to_output": False,
             "show_progress": True,
         },
         {
-            "targets": [13],
+            "targets": [35],
             "trigger": "click",
-            "inputs": [2, 11],
-            "outputs": [12],
+            "inputs": [24, 33],
+            "outputs": [34],
             "backend_fn": True,
             "js": None,
-            "status_tracker": None,
             "queue": None,
             "api_name": None,
             "scroll_to_output": False,
@@ -200,10 +207,9 @@ XRAY_CONFIG = {
             "targets": [],
             "trigger": "load",
             "inputs": [],
-            "outputs": [14],
+            "outputs": [36],
             "backend_fn": True,
-            "js": False,
-            "status_tracker": None,
+            "js": None,
             "queue": None,
             "api_name": None,
             "scroll_to_output": False,
@@ -360,6 +366,16 @@ XRAY_CONFIG_DIFF_IDS = {
                 "style": {},
             },
         },
+        {
+            "id": 37,
+            "type": "form",
+            "props": {"type": "form", "visible": True, "style": {}},
+        },
+        {
+            "id": 38,
+            "type": "form",
+            "props": {"type": "form", "visible": True, "style": {}},
+        },
     ],
     "theme": "default",
     "css": None,
@@ -368,7 +384,7 @@ XRAY_CONFIG_DIFF_IDS = {
         "id": 0,
         "children": [
             {"id": 1},
-            {"id": 22},
+            {"id": 37, "children": [{"id": 22}]},
             {
                 "id": 3,
                 "children": [
@@ -388,7 +404,7 @@ XRAY_CONFIG_DIFF_IDS = {
                     },
                 ],
             },
-            {"id": 141},
+            {"id": 38, "children": [{"id": 141}]},
         ],
     },
     "dependencies": [
@@ -399,7 +415,6 @@ XRAY_CONFIG_DIFF_IDS = {
             "outputs": [7],
             "backend_fn": True,
             "js": None,
-            "status_tracker": None,
             "queue": None,
             "api_name": None,
             "scroll_to_output": False,
@@ -412,7 +427,6 @@ XRAY_CONFIG_DIFF_IDS = {
             "outputs": [12],
             "backend_fn": True,
             "js": None,
-            "status_tracker": None,
             "queue": None,
             "api_name": None,
             "scroll_to_output": False,
@@ -424,8 +438,7 @@ XRAY_CONFIG_DIFF_IDS = {
             "inputs": [],
             "outputs": [141],
             "backend_fn": True,
-            "js": False,
-            "status_tracker": None,
+            "js": None,
             "queue": None,
             "api_name": None,
             "scroll_to_output": False,
@@ -498,7 +511,6 @@ XRAY_CONFIG_WITH_MISTAKE = {
             "id": 7,
             "type": "json",
             "props": {
-                "value": None,
                 "name": "json",
                 "style": {},
             },
@@ -545,7 +557,6 @@ XRAY_CONFIG_WITH_MISTAKE = {
             "id": 12,
             "type": "json",
             "props": {
-                "value": None,
                 "name": "json",
                 "style": {},
             },
@@ -605,7 +616,6 @@ XRAY_CONFIG_WITH_MISTAKE = {
             "trigger": "click",
             "inputs": [2, 6],
             "outputs": [7],
-            "status_tracker": None,
             "api_name": None,
             "scroll_to_output": False,
             "show_progress": True,
@@ -615,7 +625,6 @@ XRAY_CONFIG_WITH_MISTAKE = {
             "trigger": "click",
             "inputs": [2, 11],
             "outputs": [12],
-            "status_tracker": None,
             "api_name": None,
             "scroll_to_output": False,
             "show_progress": True,
