@@ -11,10 +11,22 @@ import os
 import pkgutil
 import random
 import warnings
-from copy import deepcopy
+from contextlib import contextmanager
 from distutils.version import StrictVersion
 from enum import Enum
-from typing import TYPE_CHECKING, Any, Callable, Dict, Generator, NewType, Tuple, Type
+from numbers import Number
+from pathlib import Path
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Callable,
+    Dict,
+    Generator,
+    List,
+    NewType,
+    Tuple,
+    Type,
+)
 
 import aiohttp
 import analytics
@@ -26,7 +38,6 @@ from pydantic import BaseModel, Json, parse_obj_as
 import gradio
 
 if TYPE_CHECKING:  # Only import for type checking (is False at runtime).
-    from gradio import Blocks, Interface
     from gradio.blocks import BlockContext
     from gradio.components import Component
 
@@ -185,7 +196,7 @@ def launch_counter() -> None:
         pass
 
 
-def get_default_args(func: Callable) -> Dict[str, Any]:
+def get_default_args(func: Callable) -> List[Any]:
     signature = inspect.signature(func)
     return [
         v.default if v.default is not inspect.Parameter.empty else None
@@ -215,38 +226,32 @@ def assert_configs_are_equivalent_besides_ids(
         config2["components"]
     ), "# of components are different"
 
-    mapping = {}
-
-    for c1, c2 in zip(config1["components"], config2["components"]):
-        c1, c2 = deepcopy(c1), deepcopy(c2)
-        mapping[c1["id"]] = c2["id"]
+    def assert_same_components(config1_id, config2_id):
+        c1 = list(filter(lambda c: c["id"] == config1_id, config1["components"]))[0]
+        c2 = list(filter(lambda c: c["id"] == config2_id, config2["components"]))[0]
+        c1 = copy.deepcopy(c1)
         c1.pop("id")
+        c2 = copy.deepcopy(c2)
         c2.pop("id")
         assert c1 == c2, f"{c1} does not match {c2}"
 
-    def same_children_recursive(children1, chidren2, mapping):
+    def same_children_recursive(children1, chidren2):
         for child1, child2 in zip(children1, chidren2):
-            assert (
-                mapping[child1["id"]] == child2["id"]
-            ), f"{child1} does not match {child2}"
+            assert_same_components(child1["id"], child2["id"])
             if "children" in child1 or "children" in child2:
-                same_children_recursive(child1["children"], child2["children"], mapping)
+                same_children_recursive(child1["children"], child2["children"])
 
     children1 = config1["layout"]["children"]
     children2 = config2["layout"]["children"]
-    same_children_recursive(children1, children2, mapping)
+    same_children_recursive(children1, children2)
 
     for d1, d2 in zip(config1["dependencies"], config2["dependencies"]):
         for t1, t2 in zip(d1.pop("targets"), d2.pop("targets")):
-            assert mapping[t1] == t2, f"{d1} does not match {d2}"
+            assert_same_components(t1, t2)
         for i1, i2 in zip(d1.pop("inputs"), d2.pop("inputs")):
-            assert mapping[i1] == i2, f"{d1} does not match {d2}"
+            assert_same_components(i1, i2)
         for o1, o2 in zip(d1.pop("outputs"), d2.pop("outputs")):
-            assert mapping[o1] == o2, f"{d1} does not match {d2}"
-
-        # status tracker is popped since we allow it to have different ids
-        d1.pop("status_tracker", None)
-        d2.pop("status_tracker", None)
+            assert_same_components(o1, o2)
 
         assert d1 == d2, f"{d1} does not match {d2}"
 
@@ -270,13 +275,15 @@ def format_ner_list(input_string: str, ner_groups: Dict[str : str | int]):
     return output
 
 
-def delete_none(_dict):
+def delete_none(_dict, skip_value=False):
     """
     Delete None values recursively from all of the dictionaries, tuples, lists, sets.
     Credit: https://stackoverflow.com/a/66127889/5209347
     """
     if isinstance(_dict, dict):
         for key, value in list(_dict.items()):
+            if skip_value and key == "value":
+                continue
             if isinstance(value, (list, dict, tuple, set)):
                 _dict[key] = delete_none(value)
             elif value is None or key is None:
@@ -288,7 +295,7 @@ def delete_none(_dict):
     return _dict
 
 
-def resolve_singleton(_list):
+def resolve_singleton(_list: List[Any] | Any) -> Any:
     if len(_list) == 1:
         return _list[0]
     else:
@@ -337,6 +344,8 @@ def synchronize_async(func: Callable, *args, **kwargs):
     """
     Runs async functions in sync scopes.
 
+    Can be used in any scope. See run_coro_in_background for more details.
+
     Example:
         if inspect.iscoroutinefunction(block_fn.fn):
             predictions = utils.synchronize_async(block_fn.fn, *processed_input)
@@ -353,6 +362,15 @@ def run_coro_in_background(func: Callable, *args, **kwargs):
     """
     Runs coroutines in background.
 
+    Warning, be careful to not use this function in other than FastAPI scope, because the event_loop has not started yet.
+    You can use it in any scope reached by FastAPI app.
+
+    correct scope examples: endpoints in routes, Blocks.process_api
+    incorrect scope examples: Blocks.launch
+
+    Use startup_events in routes.py if you need to run a coro in background in Blocks.launch().
+
+
     Example:
         utils.run_coro_in_background(fn, *args, **kwargs)
 
@@ -365,10 +383,7 @@ def run_coro_in_background(func: Callable, *args, **kwargs):
 
     """
     event_loop = asyncio.get_event_loop()
-    _ = event_loop.create_task(func(*args, **kwargs))
-
-
-client = httpx.AsyncClient()
+    return event_loop.create_task(func(*args, **kwargs))
 
 
 class Request:
@@ -391,6 +406,7 @@ class Request:
     """
 
     ResponseJson = NewType("ResponseJson", Json)
+    client = httpx.AsyncClient()
 
     class Method(str, Enum):
         """
@@ -461,7 +477,7 @@ class Request:
         """
         try:
             # Send the request and get the response.
-            self._response: httpx.Response = await client.send(self._request)
+            self._response: httpx.Response = await Request.client.send(self._request)
             # Raise for _status
             self._status = self._response.status_code
             if self._raise_for_status:
@@ -585,3 +601,81 @@ class Request:
     @property
     def status(self):
         return self._status
+
+
+@contextmanager
+def set_directory(path: Path):
+    """Context manager that sets the working directory to the given path."""
+    origin = Path().absolute()
+    try:
+        os.chdir(path)
+        yield
+    finally:
+        os.chdir(origin)
+
+
+def strip_invalid_filename_characters(filename: str, max_size: int = 200) -> str:
+    return ("".join([char for char in filename if char.isalnum() or char in "._- "]))[
+        :max_size
+    ]
+
+
+def sanitize_value_for_csv(value: str | Number) -> str | Number:
+    """
+    Sanitizes a value that is being written to a CSV file to prevent CSV injection attacks.
+    Reference: https://owasp.org/www-community/attacks/CSV_Injection
+    """
+    if isinstance(value, Number):
+        return value
+    unsafe_prefixes = ["=", "+", "-", "@", "\t", "\n"]
+    unsafe_sequences = [",=", ",+", ",-", ",@", ",\t", ",\n"]
+    if any(value.startswith(prefix) for prefix in unsafe_prefixes) or any(
+        sequence in value for sequence in unsafe_sequences
+    ):
+        value = "'" + value
+    return value
+
+
+def sanitize_list_for_csv(
+    values: List[str | Number] | List[List[str | Number]],
+) -> List[str | Number] | List[List[str | Number]]:
+    """
+    Sanitizes a list of values (or a list of list of values) that is being written to a
+    CSV file to prevent CSV injection attacks.
+    """
+    sanitized_values = []
+    for value in values:
+        if isinstance(value, list):
+            sanitized_value = [sanitize_value_for_csv(v) for v in value]
+            sanitized_values.append(sanitized_value)
+        else:
+            sanitized_value = sanitize_value_for_csv(value)
+            sanitized_values.append(sanitized_value)
+    return sanitized_values
+
+
+def append_unique_suffix(name: str, list_of_names: List[str]):
+    """Appends a numerical suffix to `name` so that it does not appear in `list_of_names`."""
+    list_of_names = set(list_of_names)  # for O(1) lookup
+    if name not in list_of_names:
+        return name
+    else:
+        suffix_counter = 1
+        new_name = name + f"_{suffix_counter}"
+        while new_name in list_of_names:
+            suffix_counter += 1
+            new_name = name + f"_{suffix_counter}"
+        return new_name
+
+
+def validate_url(possible_url: str) -> bool:
+    try:
+        if requests.get(possible_url).status_code == 200:
+            return True
+    except Exception:
+        pass
+    return False
+
+
+def is_update(val):
+    return type(val) is dict and "update" in val.get("__type__", "")
