@@ -3,8 +3,11 @@ import sys
 import unittest
 import unittest.mock as mock
 from contextlib import contextmanager
+from functools import partial
+from string import capwords
 
 import mlflow
+import pytest
 import requests
 import wandb
 from fastapi.testclient import TestClient
@@ -48,6 +51,14 @@ class TestInterface(unittest.TestCase):
         with self.assertRaises(TypeError):
             Interface(lambda x: x, examples=1234)
 
+    def test_partial_functions(self):
+        def greet(name, formatter):
+            return formatter("Hello " + name + "!")
+
+        greet_upper_case = partial(greet, formatter=capwords)
+        demo = Interface(fn=greet_upper_case, inputs="text", outputs="text")
+        assert demo("abubakar") == "Hello Abubakar!"
+
     def test_examples_valid_path(self):
         path = os.path.join(
             os.path.dirname(__file__), "../gradio/test_data/flagged_with_log"
@@ -104,9 +115,12 @@ class TestInterface(unittest.TestCase):
         self.assertTrue(prediction_fn.__name__ in repr[0])
         self.assertEqual(len(repr[0]), len(repr[1]))
 
-    def test_interface_none_interp(self):
+    @pytest.mark.asyncio
+    async def test_interface_none_interp(self):
         interface = Interface(lambda x: x, "textbox", "label", interpretation=[None])
-        scores = interface.interpret(["quickest brown fox"])[0]["interpretation"]
+        scores = (await interface.interpret(["quickest brown fox"]))[0][
+            "interpretation"
+        ]
         self.assertIsNone(scores)
 
     @mock.patch("webbrowser.open")
@@ -196,7 +210,7 @@ class TestTabbedInterface(unittest.TestCase):
         interface1 = Interface(lambda x: x, "textbox", "textbox")
         interface2 = Interface(lambda x: x, "image", "image")
 
-        with Blocks() as demo:
+        with Blocks(mode="tabbed_interface") as demo:
             with Tabs():
                 with TabItem(label="tab1"):
                     interface1.render()
