@@ -1,15 +1,17 @@
 import copy
+import ipaddress
+import json
 import os
 import unittest
 import unittest.mock as mock
 import warnings
-from typing import Literal
 
 import pytest
 import pytest_asyncio
 import requests
-from httpx import AsyncClient
+from httpx import AsyncClient, Response
 from pydantic import BaseModel
+from typing_extensions import Literal
 
 from gradio.test_data.blocks_configs import (
     XRAY_CONFIG,
@@ -18,6 +20,7 @@ from gradio.test_data.blocks_configs import (
 )
 from gradio.utils import (
     Request,
+    append_unique_suffix,
     assert_configs_are_equivalent_besides_ids,
     colab_check,
     delete_none,
@@ -25,9 +28,10 @@ from gradio.utils import (
     format_ner_list,
     get_local_ip_address,
     ipython_check,
-    json,
     launch_analytics,
     readme_to_html,
+    sanitize_list_for_csv,
+    sanitize_value_for_csv,
     version_check,
 )
 
@@ -99,12 +103,14 @@ class TestUtils(unittest.TestCase):
 
 
 class TestIPAddress(unittest.TestCase):
-    # def test_get_ip(self): # Removed test because internet is flaky on circleci
-    #     ip = get_local_ip_address()
-    #     try:  # check whether ip is valid
-    #         ipaddress.ip_address(ip)
-    #     except ValueError:
-    #         self.fail("Invalid IP address")
+    def test_get_ip(self):
+        ip = get_local_ip_address()
+        if ip == "No internet connection":
+            return
+        try:  # check whether ip is valid
+            ipaddress.ip_address(ip)
+        except ValueError:
+            self.fail("Invalid IP address")
 
     @mock.patch("requests.get")
     def test_get_ip_without_internet(self, mock_get):
@@ -190,7 +196,6 @@ class TestAssertConfigsEquivalent(unittest.TestCase):
                     "outputs": [2],
                     "backend_fn": True,
                     "js": None,
-                    "status_tracker": None,
                     "queue": None,
                     "api_name": "greet",
                     "scroll_to_output": False,
@@ -254,7 +259,7 @@ async def client():
     A fixture to mock the async client object.
     """
     async with AsyncClient() as mock_client:
-        with mock.patch("gradio.utils.client", mock_client):
+        with mock.patch("gradio.utils.Request.client", mock_client):
             yield
 
 
@@ -300,7 +305,7 @@ class TestRequest:
     @pytest.mark.asyncio
     async def test_validate_and_fail_with_model(self):
         class TestModel(BaseModel):
-            name: Literal[str] = "John"
+            name: Literal["John"] = "John"
             job: str
 
         client_response: Request = await Request(
@@ -314,57 +319,194 @@ class TestRequest:
         assert client_response.has_exception is True
         assert isinstance(client_response.exception, Exception)
 
-    @mock.patch("gradio.utils.Request._validate_response_data")
-    @pytest.mark.asyncio
-    async def test_exception_type(self, validate_response_data):
-        class ResponseValidationException(Exception):
-            message = "Response object is not valid."
 
-        validate_response_data.side_effect = Exception()
+def make_mock_response(return_value):
+    return Response(201, json=return_value)
 
-        client_response: Request = await Request(
-            method=Request.Method.GET,
-            url="https://reqres.in/api/users",
-            exception_type=ResponseValidationException,
-        )
-        assert isinstance(client_response.exception, ResponseValidationException)
 
-    @pytest.mark.asyncio
-    async def test_validate_with_function(self):
-        def has_name(response):
-            if response["name"] is not None:
+MOCK_REQUEST_URL = "https://very_real_url.com"
+
+
+@pytest.mark.asyncio
+async def test_get(respx_mock):
+    respx_mock.get(MOCK_REQUEST_URL).mock(
+        make_mock_response({"Host": "headers.jsontest.com"})
+    )
+
+    client_response: Request = await Request(
+        method=Request.Method.GET,
+        url=MOCK_REQUEST_URL,
+    )
+    validated_data = client_response.get_validated_data()
+    assert client_response.is_valid() is True
+    assert validated_data["Host"] == "headers.jsontest.com"
+
+
+@pytest.mark.asyncio
+async def test_post(respx_mock):
+
+    payload = {"name": "morpheus", "job": "leader"}
+    respx_mock.post(MOCK_REQUEST_URL).mock(make_mock_response(payload))
+
+    client_response: Request = await Request(
+        method=Request.Method.POST,
+        url=MOCK_REQUEST_URL,
+        json=payload,
+    )
+    validated_data = client_response.get_validated_data()
+    assert client_response.status == 201
+    assert validated_data["job"] == "leader"
+    assert validated_data["name"] == "morpheus"
+
+
+@pytest.mark.asyncio
+async def test_validate_with_model(respx_mock):
+
+    response = make_mock_response(
+        {
+            "name": "morpheus",
+            "id": "1",
+            "job": "leader",
+            "createdAt": "2",
+        }
+    )
+    respx_mock.post(MOCK_REQUEST_URL).mock(response)
+
+    class TestModel(BaseModel):
+        name: str
+        job: str
+        id: str
+        createdAt: str
+
+    client_response: Request = await Request(
+        method=Request.Method.POST,
+        url=MOCK_REQUEST_URL,
+        json={"name": "morpheus", "job": "leader"},
+        validation_model=TestModel,
+    )
+    assert isinstance(client_response.get_validated_data(), TestModel)
+
+
+@pytest.mark.asyncio
+async def test_validate_and_fail_with_model(respx_mock):
+    class TestModel(BaseModel):
+        name: Literal[str] = "John"
+        job: str
+
+    payload = {"name": "morpheus", "job": "leader"}
+    respx_mock.post(MOCK_REQUEST_URL).mock(make_mock_response(payload))
+
+    client_response: Request = await Request(
+        method=Request.Method.POST,
+        url=MOCK_REQUEST_URL,
+        json=payload,
+        validation_model=TestModel,
+    )
+    with pytest.raises(Exception):
+        client_response.is_valid(raise_exceptions=True)
+    assert client_response.has_exception is True
+    assert isinstance(client_response.exception, Exception)
+
+
+@mock.patch("gradio.utils.Request._validate_response_data")
+@pytest.mark.asyncio
+async def test_exception_type(validate_response_data, respx_mock):
+    class ResponseValidationException(Exception):
+        message = "Response object is not valid."
+
+    validate_response_data.side_effect = Exception()
+
+    respx_mock.get(MOCK_REQUEST_URL).mock(Response(201))
+
+    client_response: Request = await Request(
+        method=Request.Method.GET,
+        url=MOCK_REQUEST_URL,
+        exception_type=ResponseValidationException,
+    )
+    assert isinstance(client_response.exception, ResponseValidationException)
+
+
+@pytest.mark.asyncio
+async def test_validate_with_function(respx_mock):
+
+    respx_mock.post(MOCK_REQUEST_URL).mock(
+        make_mock_response({"name": "morpheus", "id": 1})
+    )
+
+    def has_name(response):
+        if response["name"] is not None:
+            return response
+        raise Exception
+
+    client_response: Request = await Request(
+        method=Request.Method.POST,
+        url=MOCK_REQUEST_URL,
+        json={"name": "morpheus", "job": "leader"},
+        validation_function=has_name,
+    )
+    validated_data = client_response.get_validated_data()
+    assert client_response.is_valid() is True
+    assert validated_data["id"] is not None
+    assert client_response.exception is None
+
+
+@pytest.mark.asyncio
+async def test_validate_and_fail_with_function(respx_mock):
+    def has_name(response):
+        if response["name"] is not None:
+            if response["name"] == "Alex":
                 return response
-            raise Exception
+        raise Exception
 
-        client_response: Request = await Request(
-            method=Request.Method.POST,
-            url="https://reqres.in/api/users",
-            json={"name": "morpheus", "job": "leader"},
-            validation_function=has_name,
-        )
-        validated_data = client_response.get_validated_data()
-        assert client_response.is_valid() is True
-        assert validated_data["id"] is not None
-        assert client_response.exception is None
+    respx_mock.post(MOCK_REQUEST_URL).mock(make_mock_response({"name": "morpheus"}))
 
-    @pytest.mark.asyncio
-    async def test_validate_and_fail_with_function(self):
-        def has_name(response):
-            if response["name"] is not None:
-                if response["name"] == "Alex":
-                    return response
-            raise Exception
+    client_response: Request = await Request(
+        method=Request.Method.POST,
+        url=MOCK_REQUEST_URL,
+        json={"name": "morpheus", "job": "leader"},
+        validation_function=has_name,
+    )
+    assert client_response.is_valid() is False
+    with pytest.raises(Exception):
+        client_response.is_valid(raise_exceptions=True)
+    assert client_response.exception is not None
 
-        client_response: Request = await Request(
-            method=Request.Method.POST,
-            url="https://reqres.in/api/users",
-            json={"name": "morpheus", "job": "leader"},
-            validation_function=has_name,
-        )
-        assert client_response.is_valid() is False
-        with pytest.raises(Exception):
-            client_response.is_valid(raise_exceptions=True)
-        assert client_response.exception is not None
+
+class TestSanitizeForCSV:
+    def test_unsafe_value(self):
+        assert sanitize_value_for_csv("=OPEN()") == "'=OPEN()"
+        assert sanitize_value_for_csv("=1+2") == "'=1+2"
+        assert sanitize_value_for_csv('=1+2";=1+2') == "'=1+2\";=1+2"
+
+    def test_safe_value(self):
+        assert sanitize_value_for_csv(4) == 4
+        assert sanitize_value_for_csv(-44.44) == -44.44
+        assert sanitize_value_for_csv("1+1=2") == "1+1=2"
+        assert sanitize_value_for_csv("1aaa2") == "1aaa2"
+
+    def test_list(self):
+        assert sanitize_list_for_csv([4, "def=", "=gh+ij"]) == [4, "def=", "'=gh+ij"]
+        assert sanitize_list_for_csv(
+            [["=abc", "def", "gh,+ij"], ["abc", "=def", "+ghij"]]
+        ) == [["'=abc", "def", "'gh,+ij"], ["abc", "'=def", "'+ghij"]]
+        assert sanitize_list_for_csv([1, ["ab", "=de"]]) == [1, ["ab", "'=de"]]
+
+
+class TestAppendUniqueSuffix:
+    def test_no_suffix(self):
+        name = "test"
+        list_of_names = ["test_1", "test_2"]
+        assert append_unique_suffix(name, list_of_names) == name
+
+    def test_first_suffix(self):
+        name = "test"
+        list_of_names = ["test", "test_-1"]
+        assert append_unique_suffix(name, list_of_names) == "test_1"
+
+    def test_later_suffix(self):
+        name = "test"
+        list_of_names = ["test", "test_1", "test_2", "test_3"]
+        assert append_unique_suffix(name, list_of_names) == "test_4"
 
 
 if __name__ == "__main__":
